import unittest
from unittest.mock import patch

from tmc import points
from tmc.utils import load, load_module, reload_module, get_stdout, check_source
from functools import reduce
import os
import textwrap
from random import choice, randint

exercise = 'src.read_input'
function = 'read_input'


@points('1.read_input')
class ReadInputTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        with patch('builtins.input', side_effect=list(range(1000))):
           cls.module = load_module(exercise, 'en')

    def test_0_main_ok(self):
        ok, line = check_source(self.module)
        message = """Koden för att testa funktionerna ska placeras inuti
if __name__ == "__main__":
block. Följande rad ska flyttas:
"""
        self.assertTrue(ok, message+line)
               
    def test_1_function_exists(self):
        try:
            from src.read_input import read_input
        except:
            self.fail('Din kod bör innehålla funktion read_input(prompt: str, lower_limit: int, upper_limit: int)')

    def test_2_ok_numbers(self):
        with patch('builtins.input', side_effect=["7"]):
            read_input = load(exercise, function, 'en')
            try:
                result = read_input("Enter a number:", 2, 10)
            except:
                self.fail('Se till att den funktionen kan anropas som: read_input("Enter a number:", 2, 10)')

            self.assertEqual(result, 7, "Funktionen ska returnera värde 7 när användaren anger 7")

    def test_3_too_small1(self):
        with patch('builtins.input', side_effect=["4","6"]):
            read_input = load(exercise, function, 'en')
            result = read_input("Enter a number", 5, 10)
            output = get_stdout()
            self.assertTrue("Du måste skriva in ett heltal mellan 5 och 10" in output,
                "Funktion bör ge ett felmeddelande \n'Du måste skriva in ett heltal mellan 5 och 10'\n, när den anropas som read_input('Enter a number', 5, 10) och angivet värdet är < 5.")
            self.assertEqual(result, 6, f"Funktionen ska returnera 6 när användaren anger\n4\n6\noch funktionen anropas med parametrar ('Enter a number', 5, 10). Nu returnerar funktionen {result}")

    def test_3_too_small2(self):
        with patch('builtins.input', side_effect=["2","4","6"]):
            read_input = load(exercise, function, 'en')
            result = read_input("Enter a number", 5, 10)
            output = get_stdout()
            self.assertTrue("Du måste skriva in ett heltal mellan 5 och 10" in output,
                "Funktion bör ge ett felmeddelande \n'Du måste skriva in ett heltal mellan 5 och 10'\n, när den anropas som read_input('Enter a number', 5, 10) och angivet värdet är < 5.")
            self.assertEqual(result, 6, f"Funktionen ska returnera 6 när användaren anger\n2\n4\n6\noch funktionen anropas med parametrar ('Enter a number', 5, 10). Nu returnerar funktionen {result}")

    def test_4_too_large1(self):
         with patch('builtins.input', side_effect="6 10 7 30 40 4".split()):
            read_input = load(exercise, function, 'en')
            result = read_input("Enter a number", 1, 5)
            output = get_stdout()
            self.assertTrue("Du måste skriva in ett heltal mellan 1 och 5" in output,
                "Funktion bör ge ett felmeddelande \n'Du måste skriva in ett heltal mellan 1 och 5'\n, när den anropas som read_input('Enter a number', 1, 5) och angivet värdet är > 5.")
            self.assertEqual(result, 4, f"Function should return value 4 when user enters\n6\n10\n7\n30\n40\n4\noch funktionen anropas med parametrar ('Enter a number', 1, 5). Nu returnerar funktionen {result}")

    def test_5_no_numbers(self):
        with patch('builtins.input', side_effect=["one","two","three","100"]):
            read_input = load(exercise, function, 'en')
            try:
                result = read_input("Give a number:", 95, 105)
            except:
                self.fail("Se till att ditt program kan köras med input\none\ntwo\nthree\n100")
            output = get_stdout()
            self.assertTrue("Du måste skriva in ett heltal mellan 95 och 105" in output,
                "Funktion bör ge ett felmeddelande \n'Du måste skriva in ett heltal mellan 95 och  105', when its called with parameters ('Give a number', 95, 105) och indata innehåller endast bokstäver.")
            self.assertEqual(result, 100, f"Function should return value 100 when user enters 100 och the function is called with arguments ('Enter a number', 95, 105). Nu returnerar funktionen {result}")

if __name__ == '__main__':
    unittest.main()
